;;; erc-extcmd.el --- extensions to erc
;; Copyright (C) 2006,2008 Patrik Nilsson 

;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation, either version 3 of the License, or
;; (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

;; rev: 1
;; History:
;; before rev 0 isn't documented
;; 
;; rev 0:
;; rewrote all effected functions to use my wrapper for erc-display-message
;; and added /rot a rot13 sender
;;
;; rev 1:
;; added /keyword works exactly as /pal, maybe they should share some code..
;; added /savepik that saves Pals, Ignore and keywords to a file that can be
;; loaded as normal elisp

;; some global functions etc...
(defun my-erc-display-message( line )
  "a wrapper around erc-display-message"
  (erc-display-message
   nil 'notice (current-buffer)
   line))

;; erc-extcmd's

(defun erc-cmd-OPER (nick password &rest ignore)
  "Login as a adminstrator of the server."
  (erc-server-send (format "OPER %s %s" nick password)))

(defun erc-cmd-USERCOUNT (&rest ignore)
  "Counts the number of users in the channel"
  (my-erc-display-message
   (format "%d users in %s"
   (length
	(erc-get-channel-user-list))
   (current-buffer))))

;; NOTE: it doesn't work if we have 13 in the name...
(defun erc-cmd-ROT (&rest words)
  "sends the line trough rot13 then into the current channel"
  (erc-send-message (rot13-string
                (mapconcat 'identity words " "))))

;; bip related
(defun erc-cmd-BIP (command &rest ignore)
  "An alias to send commands to bip"
  (if (string-equal command "help")
      (my-erc-display-message 
       "BIP: known bip commands: reload, list, jump, blreset" )
    (erc-server-send (format "BIP %s" command))))

;; pal & keyword related
(defun erc-cmd-PAL (&optional user &rest ignore)
  "adds a nick to the pal list, if its already added remove it."
  (if user
      (if (and (not (cdr (erc-list-match erc-pals user)))
               (not (string-equal (car erc-pals) user)))
          (progn ;; add
            (my-erc-display-message
             (format "%s is now a registered pal" user))
            (add-to-list 'erc-pals user))
        (progn ;; remove
          (my-erc-display-message
           (format "%s isn't a pal anymore" user))
          (setq erc-pals (delete user erc-pals)))
        )
    ;; no user specified (this should be hacked some more so its output is nicer)
    (progn ;; the following is sort of stolen from erc-cmd-IGNORE
      (my-erc-display-message
       "Pals:")
      (mapc #'(lambda (item)
                (my-erc-display-message
                 item))
            (with-current-buffer (current-buffer) erc-pals)))))

;; keyword related
(defun erc-cmd-KEYWORD (&optional keyword &rest ignore)
  "adds a keyword to the keyword list, if its already added remove it."
  (if keyword
      (if (and (not (cdr (erc-list-match erc-keywords keyword)))
               (not (string-equal (car erc-keywords) keyword)))
          (progn ;; add
            (my-erc-display-message
             (format "%s is now a registered keyword" keyword))
            (add-to-list 'erc-keywords keyword))
        (progn ;; remove
          (my-erc-display-message
           (format "%s isn't a registered keyword anymore" keyword))
          (setq erc-keywords (delete keyword erc-keywords)))
        )
    ; Hack a nicer output...
    (progn ;; the following is sort of stolen from erc-cmd-IGNORE
      (my-erc-display-message
       "Keywords:")
      (mapc #'(lambda (item)
                (my-erc-display-message
                 item))
            (with-current-buffer (current-buffer) erc-keywords)))))

;; pal/ignore/keywords save/Load saved lists
; saving
(defun erc-cmd-SAVEPIK (&rest ignore)
  "saves Pals, Ignore and Keywords to a file, that one can load
   on startup, currently this file is hardcoded to ~/elisp/erc-pik.el"
  ; simpler version
  (with-temp-buffer
    (insert (concat
             ";; This is a autogenerated file by erc-extcmd.el"
             "(setq erc-pals '" (prin1-to-string erc-pals) ")"
             "\n"
             "(setq erc-keywords '" (prin1-to-string erc-keywords) ")"
             "\n"
             "(setq erc-ignore-list '" (prin1-to-string erc-ignore-list)
             ")"))
    
    (when (file-writable-p "~/elisp/erc-pik.el") ; make customizable
      (write-region (point-min) (point-max)
                    "~/elisp/erc-pik.el"))))

(provide 'erc-extcmd)
